#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <stdarg.h>

#include "glut/glut.h"
#include "gl_texture.h"
#include "tga_image.h"
#include "vector_2d.h"

#pragma comment( linker, "/entry:\"mainCRTStartup\"" )  // set the entry point to be main()


// menu
enum MenuCommands {
	MENU_EXIT = 0
};


struct point {

	double x, y;
	point* next;

};

struct test_pixel {

	bool active;
	point* point_list;
};

vector_2d* v;
vector_2d p1;

char* image_file_name = NULL;

GLsizei iWidth = 300.0, iHeight = 300.0;


gl_texture* texture;
gl_texture* test_texture; 

tga_image* image;

bool images_ready = false;

GLvoid *font_style = GLUT_BITMAP_HELVETICA_10;

void setfont(char* name, int size) {

    font_style = GLUT_BITMAP_HELVETICA_10;
    if (strcmp(name, "helvetica") == 0) {
        if (size == 12) 
            font_style = GLUT_BITMAP_HELVETICA_12;
        else if (size == 18)
            font_style = GLUT_BITMAP_HELVETICA_18;
    } else if (strcmp(name, "times roman") == 0) {
        font_style = GLUT_BITMAP_TIMES_ROMAN_10;
        if (size == 24)
            font_style = GLUT_BITMAP_TIMES_ROMAN_24;
    } else if (strcmp(name, "8x13") == 0) {
        font_style = GLUT_BITMAP_8_BY_13;
    } else if (strcmp(name, "9x15") == 0) {
        font_style = GLUT_BITMAP_9_BY_15;
    }
}


void drawstr(GLuint x, GLuint y, char* format, ...) {

    va_list args;
    char buffer[255], *s;
    
    va_start(args, format);
    vsprintf(buffer, format, args);
    va_end(args);
    
    glRasterPos2i(x, y);
    for (s = buffer; *s; s++) {
        glutBitmapCharacter(font_style, *s);
	}
}


void init() {


	unsigned char* image_ptr = image->get_ptr_image_data();

	int h = image->get_height();
	int w = image->get_width();

	int i=0, j=0;

	p1.x = image->get_width()/4;
	p1.y = image->get_height()/4;

	// set origin of vector field
	vector_2d origin(-1,-1);


	// note that image is in a "flat" format: access
	// into a particular [row, column] in the image is
	// given by image_width*row_number + column_number

	// precompute r (distance from origin) vectors
	vector_2d* r = new vector_2d[h*w];
	for(i=0;i<h;i++) {
		for(j=0;j<w;j++) {
			// r value for a particular pixel is given by
			// pixel coords - origin coords
			r[w*i+j] = vector_2d(j, i) - origin;
		}
	}

	// create the vector force field
	v = new vector_2d[h*w];

	for(i=0;i<h*w;i++) {


		// display status
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// clear screen
		drawstr(0,0,"Computing field: %.2f percent complete\n", (float)100*i/(image->get_width()*image->get_height()) );
		glutSwapBuffers();
		glutPostRedisplay();


		// ************************************************************************ //
		//								your code here								//
		//						fill the vector force field v[]						//
		// ************************************************************************ //

	}

	// find magnitudes in image
	// as well as min and max magnitudes in image
	double* mag = new double[h*w];
	double max = v[0].Length();
	double min = v[0].Length();
	for(i=0; i<w*h; i++) {
		mag[i] = v[i].Length();
		if(mag[i] > max) {
			max = mag[i];
		}
		if(mag[i] < min) {
			min = mag[i];
		}
	}

	// allocate space for new gray scale image
	unsigned char* image2_ptr = (unsigned char*)malloc (w*h);

	// scale data from 0 to 255 and create image
	for(i=0; i<w*h; i++) {
		image2_ptr[i] = (unsigned char)(255*mag[i]/(max));
	}

	// create textures
	texture = new gl_texture();
	test_texture = new gl_texture();

	// create texture from original image
	bool img_1_ready = texture->create_raw_texture(image_ptr, GL_LUMINANCE, w, h);
	// create texture from new image
	bool img_2_ready = test_texture->create_raw_texture(image2_ptr, GL_LUMINANCE, w, h);


	if (img_1_ready && img_2_ready) { images_ready = true; }
	else { images_ready = false; }
}


void draw_scene() {

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// clear screen

	if (images_ready == true) {

		glDisable(GL_LIGHTING);
		glPushMatrix();
		glLoadIdentity();

		gluLookAt(  0, 0, 1,
					0, 0, 0,
					0, 1, 0);

		float w = 0.5;
		float h = 1.0;
		float r = 1.0;

		glColor3f(1.0f, 1.0f, 1.0f);

		glEnable(GL_TEXTURE_2D);
		texture->use();

		glTranslated(-0.5, 0.0, 0.0);

		// draw
		glBegin(GL_QUADS);
		glTexCoord2d(0.0,	0.0);
		glVertex3f	(-w,	-h,		0);
		glTexCoord2d(r,		0.0);
		glVertex3f	(w,		-h,		0);
		glTexCoord2d(r,		r);
		glVertex3f	(w,		h,		0);
		glTexCoord2d(0.0,	r);
		glVertex3f	(-w,	h,		0);
		glTexCoord2d(0.0,	0.0);
		glVertex3f	(-w,	-h,		0);
		glEnd();

		test_texture->use();
		glTranslated(1.0, 0.0, 0.0);

		glBegin(GL_QUADS);
		glTexCoord2d(0.0,	0.0);
		glVertex3f	(-w,	-h,		0);
		glTexCoord2d(r,		0.0);
		glVertex3f	(w,		-h,		0);
		glTexCoord2d(r,		r);
		glVertex3f	(w,		h,		0);
		glTexCoord2d(0.0,	r);
		glVertex3f	(-w,	h,		0);
		glTexCoord2d(0.0,	0.0);
		glVertex3f	(-w,	-h,		0);
		glEnd();

		glDisable(GL_TEXTURE_2D);

		glPopMatrix();

		glutSwapBuffers(); 
		glutPostRedisplay();
	}

	else {

		//shift origin to left side
		glTranslated(-1.0, 0.0, 0.0);

		init();
	}
}



void handle_resize( GLsizei iWidth, GLsizei iHeight ) { 

	glViewport (0, 0, iWidth, iHeight); 
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();

	glOrtho(1000,  // left
			1000,  // right
            1000,  // bottom
            1000,  // top
            -1000, // near
            1000); // far


	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


void handle_idle() {

	//glutPostRedisplay();
}


bool load_images() {

	image = new tga_image();

	if ( image->load_TGA(image_file_name) == true ) {

		iWidth = image->get_width()*2; // displaying two images side by side
		iHeight = image->get_height();

		return true;
	}

	return false;
}


void handle_menu_select(int idCommand) {

	switch (idCommand) {
		case MENU_EXIT:
			exit(0);
			break;
	};

	glutPostRedisplay();
}


int build_menu (void) {

	int menu;

	menu = glutCreateMenu(handle_menu_select);
	glutAddMenuEntry ("Exit\tEsc", MENU_EXIT);

	return menu;
}


void handle_keyboard(unsigned char key, int x, int y) {

	switch (key) {

	case 27:	// escape
		handle_menu_select(MENU_EXIT);
		break;
	}
}


void handle_special(int key, int x, int y) {

	vector_2d temp;
	vector_2d new_pos;

	switch (key) {
	case GLUT_KEY_LEFT:
		break;
	case GLUT_KEY_RIGHT:
		break;
	case GLUT_KEY_UP :
		break;
	case GLUT_KEY_DOWN:
		break;
	case GLUT_KEY_PAGE_UP:
		break;
	case GLUT_KEY_PAGE_DOWN:
		break;
	}
}


int main(int argc, char** argv) {

	if (argc > 1) {

		image_file_name = argv[1];

		load_images();

		glutInit(&argc, argv);
		glutInitWindowPosition(0, 0);
		glutInitWindowSize(iWidth, iHeight);
		glutInitDisplayMode( GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE );
		glutCreateWindow("Image Window");

		build_menu();
		glutAttachMenu(GLUT_RIGHT_BUTTON);

		glutReshapeFunc(handle_resize);
		glutDisplayFunc(draw_scene);
		glutIdleFunc(handle_idle);
		glutKeyboardFunc(handle_keyboard);
		glutSpecialFunc(handle_special);

		handle_resize(iWidth, iHeight);

		glutMainLoop();
	}
	else {

		glutInit(&argc, argv);
		glutInitWindowPosition(0, 0);
		glutInitWindowSize(iWidth, iHeight);
		glutInitDisplayMode( GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE );
		glutCreateWindow("Image Window");

		build_menu();
		glutAttachMenu(GLUT_RIGHT_BUTTON);

		glutReshapeFunc(handle_resize);
		glutDisplayFunc(draw_scene);
		glutIdleFunc(handle_idle);
		glutKeyboardFunc(handle_keyboard);
		glutSpecialFunc(handle_special);

		handle_resize(iWidth, iHeight);

		glutMainLoop();
	}

	return(0);
}