
#include "tga_image.h"


tga_image::tga_image() {
	imageData = NULL;
}

tga_image::tga_image(char* file_name) {

	// if load fails set ptr to NULL
	if ( this->load_TGA(file_name) == false ){
		imageData = NULL;
	}
}


tga_image::~tga_image() {

	// release memory holding image data
	free(imageData);
	imageData = NULL;
}


//
// load_TGA
//
// Loads up a targa file.  Supported types are 8,24 and 32 uncompressed images.  
// id is the texture ID to bind to
//
bool tga_image::load_TGA(char* file_name) {

	unsigned char type[4];
	unsigned char info[7];	

	FILE* f_ptr;  // pointer to image file

	// open image file
	if (!(f_ptr = fopen (file_name, "r+bt"))) { return false; }
	
	fread (&type, sizeof (char), 3, f_ptr); // read in colormap info and image type, byte 0 ignored
	fseek (f_ptr, 12, SEEK_SET);			// seek past the header and useless info
	fread (&info, sizeof (char), 6, f_ptr);

	// see if the iamge is an expected type
	if (type[1] != 0 || (type[2] != 2 && type[2] != 3)) { return false; }
	
	imageWidth = info[0] + info[1] * 256; 
	imageHeight = info[2] + info[3] * 256;
	imageBits = info[4]; 

	size = imageWidth * imageHeight; 

	// make sure dimension is a power of 2 
	if (!check_size (imageWidth) || !check_size (imageHeight)) {
		return false;
	}


	if ( (imageBits != 32) && (imageBits != 24) && (imageBits != 8) ) {
		return false;
	}

	imageData = get_data (f_ptr, size, imageBits);

	// no image data
	if (imageData == NULL) { return false; }

	// close the file
	fclose (f_ptr);

	return true;
}

//
// get_ptr_image_data
// 
// gets pointer to image data
//
unsigned char* tga_image::get_ptr_image_data() {

	return(imageData);
}

int tga_image::get_width() {

	if (imageData == NULL) {
		return(0);
	}

	return(imageWidth);
}

int tga_image::get_height() {

	if (imageData == NULL) {
		return(0);
	}

	return(imageHeight);
}

int tga_image::get_format() {

	return(imageFormat);
}


//
//check_size
//
//Make sure its a power of 2
//
int tga_image::check_size (int x) {

	return 1;

	if (x == 2	 || x == 4   || 
		x == 8	 || x == 16  || 
		x == 32  || x == 64  ||
		x == 128 || x == 256 ||
		x == 512) 
	{ return 1; }

	else { return 0; }
}



//
//get_RGBA
//
//Reads in RGBA data for a 32bit image
//
unsigned char* tga_image::get_RGBA (FILE *f_ptr, int size) {

	unsigned char *rgba;
	unsigned char temp;
	int bread;
	int i;

	rgba = (unsigned char*)malloc(size * 4); 

	if (rgba == NULL)
		return 0;

	bread = fread (rgba, sizeof (unsigned char), size * 4, f_ptr); 

	/* TGA is stored in BGRA, make it RGBA */
	if (bread != size * 4)
	{
		free (rgba);
		return 0;
	}

	for (i = 0; i < size * 4; i += 4 )
	{
		temp = rgba[i];
		rgba[i] = rgba[i + 2];
		rgba[i + 2] = temp;
	}

	return rgba;
}


//
//
//get_RGB
//
//Reads in RGB data for a 24bit image. 
//
unsigned char* tga_image::get_RGB (FILE* f_ptr, int size) {

	unsigned char *rgb;
	unsigned char temp;
	int bread;
	int i;

	rgb = (unsigned char*)malloc(size * 3); 

	if (rgb == NULL)
		return 0;

	bread = fread (rgb, sizeof (unsigned char), size * 3, f_ptr);

	if (bread != size * 3) {

		free (rgb);
		return 0;
	}

	/* TGA is stored in BGR, make it RGB */
	for (i = 0; i < size * 3; i += 3) {

		temp = rgb[i];
		rgb[i] = rgb[i + 2];
		rgb[i + 2] = temp;
	}

	return rgb;
}


//get_gray
//
//Gets the grayscale image data.	Used as an alpha channel.
//
unsigned char* tga_image::get_gray (FILE *f_ptr, int size) {

	unsigned char *grayData;
	int bread;

	grayData = (unsigned char*)malloc (size);

	if (grayData == NULL)
		return 0;

	bread = fread (grayData, sizeof (unsigned char), size, f_ptr);

	if (bread != size) {

		free (grayData);
		return 0;
	}

	return grayData;
}

//
// get_data
//
// Gets the image data for the specified bit depth.
//
unsigned char* tga_image::get_data (FILE *f_ptr, int sz, int iBits) {

	if (iBits == 32) {
		imageFormat = RGBA;
		return get_RGBA (f_ptr, sz);
	}
	else if (iBits == 24) {
		imageFormat = RGB;
		return get_RGB (f_ptr, sz);
	}
	else if (iBits == 8) {
		imageFormat = LUMINANCE;
		return get_gray (f_ptr, sz);
	}
	return NULL;
}

