//	vector_2d.h
//  Written by:	 Joe Strout
//  Modified by: Dan Hand

//	This class defines a two-dimensional vector_2d, with appropriate
//	operators.

#ifndef VECTOR_2D_H
#define VECTOR_2D_H

#ifndef real
#define real double
#endif

#include <iostream.h>
#include <math.h>


class vector_2d {
   public:
	real x,y;				// vector_2d components
	
	vector_2d() { x=y=0; }			// constructor
	vector_2d(const real pX, const real pY)
	: x(pX), y(pY) {}

	// vector_2d arithmatic

	vector_2d operator+(const vector_2d pV) const
	{ vector_2d out( x+pV.x, y+pV.y); return out; }

	vector_2d operator-(const vector_2d pV) const
	{ vector_2d out( x-pV.x, y-pV.y); return out; }

	vector_2d& operator+=(const vector_2d pV)
	{ x+=pV.x; y+=pV.y; return *this; }

	vector_2d& operator-=(const vector_2d pV)
	{ x-=pV.x; y-=pV.y; return *this; }

	real Dot(const vector_2d pV) const			// dot product
	{ return x*pV.x + y*pV.y; }

	friend real Dot( const vector_2d p1, const vector_2d p2 )
	{ return p1.Dot(p2); }

	vector_2d operator*(const real pR) const		// * a scalar
	{ vector_2d out( x*pR, y*pR ); return out; }

	friend vector_2d operator*(const real pR, const vector_2d pV)
	{ vector_2d out( pV.x*pR, pV.y*pR ); return out; }

	vector_2d& operator*=(const real pR)
	{ x*=pR; y*=pR; return *this; }

	vector_2d operator/(const vector_2d pV)
	{ vector_2d out( x/pV.x, y/pV.y ); return out; }

	vector_2d operator/(const double pR)
	{ vector_2d out( x/pR, y/pR ); return out; }

	// magnitude

	real LenSqr(void) const
	{ return x*x+y*y; }

	real Length(void) const
	{ return sqrt(LenSqr()); }

	// projection (component of *this parallel to pV)
	// Note: component perpendicular to pV is:  *this - Proj(pV)

	vector_2d Proj(const vector_2d pV)
	{ vector_2d out( pV * (this->Dot(pV) / pV.LenSqr()) ); return out; }

	// cosine of the angle between two vector_2ds:
	real CosAng( const vector_2d pV)
	{ return this->Dot(pV) / (Length() * pV.Length()); }

	// comparison
	int operator==( const vector_2d pV) const
	{ return x==pV.x && y==pV.y; }

	int operator!=( const vector_2d pV) const
	{ return x!=pV.x || y!=pV.y; }

	// input/output
	friend ostream& operator<<(ostream& pStr, const vector_2d& pV)
	{ return (pStr << '(' << pV.x << ',' << pV.y << ')'); }

	friend istream& operator>>(istream pStr, vector_2d& pV)
	{ return (pStr >> pV.x >> pV.y); }
};

#endif
